@@ -44,7 +44,7 @@ typedef enum {
     XML_FROM_OUTPUT,	/* The serialization code */
     XML_FROM_IO,	/* The Input/Output stack */
     XML_FROM_FTP,	/* The FTP module */
-    XML_FROM_HTTP,	/* The FTP module */
+    XML_FROM_HTTP,	/* The HTTP module */
     XML_FROM_XINCLUDE,	/* The XInclude processing */
     XML_FROM_XPATH,	/* The XPath module */
     XML_FROM_XPOINTER,	/* The XPointer module */
@@ -57,7 +57,10 @@ typedef enum {
     XML_FROM_CATALOG,	/* The Catalog module */
     XML_FROM_C14N,	/* The Canonicalization module */
     XML_FROM_XSLT,	/* The XSLT engine from libxslt */
-    XML_FROM_VALID	/* The XML DTD validation with valid context */
+    XML_FROM_VALID,	/* The XML DTD validation with valid context */
+    XML_FROM_CHECK,	/* The error checking module */
+    XML_FROM_WRITER,	/* The xmlwriter module */
+    XML_FROM_MODULE	/* The dynamically loaded module module*/
 } xmlErrorDomain;
 
 /**
@@ -79,7 +82,7 @@ struct _xmlError {
     char       *str2;	/* extra string information */
     char       *str3;	/* extra string information */
     int		int1;	/* extra number information */
-    int		int2;	/* extra number information */
+    int		int2;	/* column number of the error or 0 if N/A (todo: rename this field when we would break ABI) */
     void       *ctxt;   /* the parser context if available */
     void       *node;   /* the node in the tree */
 };
@@ -530,122 +533,128 @@ typedef enum {
     XML_SCHEMAP_UNKNOWN_UNION_CHILD, /* 1754 */
     XML_SCHEMAP_ELEM_DEFAULT_FIXED, /* 1755 */
     XML_SCHEMAP_REGEXP_INVALID, /* 1756 */
-    XML_SCHEMAP_FAILED_LOAD, /* 1756 */
-    XML_SCHEMAP_NOTHING_TO_PARSE, /* 1757 */
-    XML_SCHEMAP_NOROOT, /* 1758 */
-    XML_SCHEMAP_REDEFINED_GROUP, /* 1759 */
-    XML_SCHEMAP_REDEFINED_TYPE, /* 1760 */
-    XML_SCHEMAP_REDEFINED_ELEMENT, /* 1761 */
-    XML_SCHEMAP_REDEFINED_ATTRGROUP, /* 1762 */
-    XML_SCHEMAP_REDEFINED_ATTR, /* 1763 */
-    XML_SCHEMAP_REDEFINED_NOTATION, /* 1764 */
-    XML_SCHEMAP_FAILED_PARSE, /* 1765 */
-    XML_SCHEMAP_UNKNOWN_PREFIX, /* 1766 */
-    XML_SCHEMAP_DEF_AND_PREFIX, /* 1767 */
-    XML_SCHEMAP_UNKNOWN_INCLUDE_CHILD, /* 1768 */
-    XML_SCHEMAP_INCLUDE_SCHEMA_NOT_URI, /* 1769 */
-    XML_SCHEMAP_INCLUDE_SCHEMA_NO_URI, /* 1770 */
-    XML_SCHEMAP_NOT_SCHEMA, /* 1771 */
-    XML_SCHEMAP_UNKNOWN_MEMBER_TYPE, /* 1772 */
-    XML_SCHEMAP_INVALID_ATTR_USE, /* 1773 */
-    XML_SCHEMAP_RECURSIVE, /* 1774 */
-    XML_SCHEMAP_SUPERNUMEROUS_LIST_ITEM_TYPE, /* 1775 */
-    XML_SCHEMAP_INVALID_ATTR_COMBINATION, /* 1776 */
-    XML_SCHEMAP_INVALID_ATTR_INLINE_COMBINATION, /* 1777 */
-    XML_SCHEMAP_MISSING_SIMPLETYPE_CHILD, /* 1778 */
-    XML_SCHEMAP_INVALID_ATTR_NAME, /* 1779 */
-    XML_SCHEMAP_REF_AND_CONTENT, /* 1780 */
-    XML_SCHEMAP_CT_PROPS_CORRECT_1, /* 1781 */
-    XML_SCHEMAP_CT_PROPS_CORRECT_2, /* 1782 */
-    XML_SCHEMAP_CT_PROPS_CORRECT_3, /* 1783 */
-    XML_SCHEMAP_CT_PROPS_CORRECT_4, /* 1784 */
-    XML_SCHEMAP_CT_PROPS_CORRECT_5, /* 1785 */
-    XML_SCHEMAP_DERIVATION_OK_RESTRICTION_1, /* 1786 */
-    XML_SCHEMAP_DERIVATION_OK_RESTRICTION_2_1_1, /* 1787 */
-    XML_SCHEMAP_DERIVATION_OK_RESTRICTION_2_1_2, /* 1788 */
-    XML_SCHEMAP_DERIVATION_OK_RESTRICTION_2_2, /* 1789 */
-    XML_SCHEMAP_DERIVATION_OK_RESTRICTION_3, /* 1790 */
-    XML_SCHEMAP_WILDCARD_INVALID_NS_MEMBER, /* 1791 */
-    XML_SCHEMAP_INTERSECTION_NOT_EXPRESSIBLE, /* 1792 */
-    XML_SCHEMAP_UNION_NOT_EXPRESSIBLE, /* 1793 */
-    XML_SCHEMAP_SRC_IMPORT_3_1, /* 1794 */
-    XML_SCHEMAP_SRC_IMPORT_3_2, /* 1795 */
-    XML_SCHEMAP_DERIVATION_OK_RESTRICTION_4_1, /* 1796 */
-    XML_SCHEMAP_DERIVATION_OK_RESTRICTION_4_2, /* 1797 */
-    XML_SCHEMAP_DERIVATION_OK_RESTRICTION_4_3, /* 1798 */
-    XML_SCHEMAP_COS_CT_EXTENDS_1_3, /* 1799 */
-    XML_SCHEMAV_NOROOT = 1800,
-    XML_SCHEMAV_UNDECLAREDELEM, /* 1801 */
-    XML_SCHEMAV_NOTTOPLEVEL, /* 1802 */
-    XML_SCHEMAV_MISSING, /* 1803 */
-    XML_SCHEMAV_WRONGELEM, /* 1804 */
-    XML_SCHEMAV_NOTYPE, /* 1805 */
-    XML_SCHEMAV_NOROLLBACK, /* 1806 */
-    XML_SCHEMAV_ISABSTRACT, /* 1807 */
-    XML_SCHEMAV_NOTEMPTY, /* 1808 */
-    XML_SCHEMAV_ELEMCONT, /* 1809 */
-    XML_SCHEMAV_HAVEDEFAULT, /* 1810 */
-    XML_SCHEMAV_NOTNILLABLE, /* 1811 */
-    XML_SCHEMAV_EXTRACONTENT, /* 1812 */
-    XML_SCHEMAV_INVALIDATTR, /* 1813 */
-    XML_SCHEMAV_INVALIDELEM, /* 1814 */
-    XML_SCHEMAV_NOTDETERMINIST, /* 1815 */
-    XML_SCHEMAV_CONSTRUCT, /* 1816 */
-    XML_SCHEMAV_INTERNAL, /* 1817 */
-    XML_SCHEMAV_NOTSIMPLE, /* 1818 */
-    XML_SCHEMAV_ATTRUNKNOWN, /* 1819 */
-    XML_SCHEMAV_ATTRINVALID, /* 1820 */
-    XML_SCHEMAV_VALUE, /* 1821 */
-    XML_SCHEMAV_FACET, /* 1822 */
-    XML_SCHEMAV_CVC_DATATYPE_VALID_1_2_1, /* 1823 */
-    XML_SCHEMAV_CVC_DATATYPE_VALID_1_2_2, /* 1824 */
-    XML_SCHEMAV_CVC_DATATYPE_VALID_1_2_3, /* 1825 */
-    XML_SCHEMAV_CVC_TYPE_3_1_1, /* 1826 */
-    XML_SCHEMAV_CVC_TYPE_3_1_2, /* 1827 */
-    XML_SCHEMAV_CVC_FACET_VALID, /* 1828 */
-    XML_SCHEMAV_CVC_LENGTH_VALID, /* 1829 */
-    XML_SCHEMAV_CVC_MINLENGTH_VALID, /* 1830 */
-    XML_SCHEMAV_CVC_MAXLENGTH_VALID, /* 1831 */
-    XML_SCHEMAV_CVC_MININCLUSIVE_VALID, /* 1832 */
-    XML_SCHEMAV_CVC_MAXINCLUSIVE_VALID, /* 1833 */
-    XML_SCHEMAV_CVC_MINEXCLUSIVE_VALID, /* 1834 */
-    XML_SCHEMAV_CVC_MAXEXCLUSIVE_VALID, /* 1835 */
-    XML_SCHEMAV_CVC_TOTALDIGITS_VALID, /* 1836 */
-    XML_SCHEMAV_CVC_FRACTIONDIGITS_VALID, /* 1837 */
-    XML_SCHEMAV_CVC_PATTERN_VALID, /* 1838 */
-    XML_SCHEMAV_CVC_ENUMERATION_VALID, /* 1839 */
-    XML_SCHEMAV_CVC_COMPLEX_TYPE_2_1, /* 1840 */
-    XML_SCHEMAV_CVC_COMPLEX_TYPE_2_2, /* 1841 */
-    XML_SCHEMAV_CVC_COMPLEX_TYPE_2_3, /* 1842 */
-    XML_SCHEMAV_CVC_COMPLEX_TYPE_2_4, /* 1843 */
-    XML_SCHEMAV_CVC_ELT_1, /* 1844 */
-    XML_SCHEMAV_CVC_ELT_2, /* 1845 */
-    XML_SCHEMAV_CVC_ELT_3_1,
-    XML_SCHEMAV_CVC_ELT_3_2_1,
-    XML_SCHEMAV_CVC_ELT_3_2_2,
-    XML_SCHEMAV_CVC_ELT_4_1,
-    XML_SCHEMAV_CVC_ELT_4_2,
-    XML_SCHEMAV_CVC_ELT_4_3,
-    XML_SCHEMAV_CVC_ELT_5_1_1,
-    XML_SCHEMAV_CVC_ELT_5_1_2,
-    XML_SCHEMAV_CVC_ELT_5_2_1,  
-    XML_SCHEMAV_CVC_ELT_5_2_2_1,
-    XML_SCHEMAV_CVC_ELT_5_2_2_2_1,
-    XML_SCHEMAV_CVC_ELT_5_2_2_2_2,
-    XML_SCHEMAV_CVC_ELT_6,
-    XML_SCHEMAV_CVC_ELT_7,
-    XML_SCHEMAV_CVC_ATTRIBUTE_1,
-    XML_SCHEMAV_CVC_ATTRIBUTE_2,
-    XML_SCHEMAV_CVC_ATTRIBUTE_3,
-    XML_SCHEMAV_CVC_ATTRIBUTE_4,
-    XML_SCHEMAV_CVC_COMPLEX_TYPE_3_1,
-    XML_SCHEMAV_CVC_COMPLEX_TYPE_3_2_1,
-    XML_SCHEMAV_CVC_COMPLEX_TYPE_3_2_2,
-    XML_SCHEMAV_CVC_COMPLEX_TYPE_4,
-    XML_SCHEMAV_CVC_COMPLEX_TYPE_5_1,
-    XML_SCHEMAV_CVC_COMPLEX_TYPE_5_2,
-    XML_SCHEMAV_ELEMENT_CONTENT, /* 1846 non-W3C */
-    XML_SCHEMAV_DOCUMENT_ELEMENT_MISSING, /* non-W3C */
+    XML_SCHEMAP_FAILED_LOAD, /* 1757 */
+    XML_SCHEMAP_NOTHING_TO_PARSE, /* 1758 */
+    XML_SCHEMAP_NOROOT, /* 1759 */
+    XML_SCHEMAP_REDEFINED_GROUP, /* 1760 */
+    XML_SCHEMAP_REDEFINED_TYPE, /* 1761 */
+    XML_SCHEMAP_REDEFINED_ELEMENT, /* 1762 */
+    XML_SCHEMAP_REDEFINED_ATTRGROUP, /* 1763 */
+    XML_SCHEMAP_REDEFINED_ATTR, /* 1764 */
+    XML_SCHEMAP_REDEFINED_NOTATION, /* 1765 */
+    XML_SCHEMAP_FAILED_PARSE, /* 1766 */
+    XML_SCHEMAP_UNKNOWN_PREFIX, /* 1767 */
+    XML_SCHEMAP_DEF_AND_PREFIX, /* 1768 */
+    XML_SCHEMAP_UNKNOWN_INCLUDE_CHILD, /* 1769 */
+    XML_SCHEMAP_INCLUDE_SCHEMA_NOT_URI, /* 1770 */
+    XML_SCHEMAP_INCLUDE_SCHEMA_NO_URI, /* 1771 */
+    XML_SCHEMAP_NOT_SCHEMA, /* 1772 */
+    XML_SCHEMAP_UNKNOWN_MEMBER_TYPE, /* 1773 */
+    XML_SCHEMAP_INVALID_ATTR_USE, /* 1774 */
+    XML_SCHEMAP_RECURSIVE, /* 1775 */
+    XML_SCHEMAP_SUPERNUMEROUS_LIST_ITEM_TYPE, /* 1776 */
+    XML_SCHEMAP_INVALID_ATTR_COMBINATION, /* 1777 */
+    XML_SCHEMAP_INVALID_ATTR_INLINE_COMBINATION, /* 1778 */
+    XML_SCHEMAP_MISSING_SIMPLETYPE_CHILD, /* 1779 */
+    XML_SCHEMAP_INVALID_ATTR_NAME, /* 1780 */
+    XML_SCHEMAP_REF_AND_CONTENT, /* 1781 */
+    XML_SCHEMAP_CT_PROPS_CORRECT_1, /* 1782 */
+    XML_SCHEMAP_CT_PROPS_CORRECT_2, /* 1783 */
+    XML_SCHEMAP_CT_PROPS_CORRECT_3, /* 1784 */
+    XML_SCHEMAP_CT_PROPS_CORRECT_4, /* 1785 */
+    XML_SCHEMAP_CT_PROPS_CORRECT_5, /* 1786 */
+    XML_SCHEMAP_DERIVATION_OK_RESTRICTION_1, /* 1787 */
+    XML_SCHEMAP_DERIVATION_OK_RESTRICTION_2_1_1, /* 1788 */
+    XML_SCHEMAP_DERIVATION_OK_RESTRICTION_2_1_2, /* 1789 */
+    XML_SCHEMAP_DERIVATION_OK_RESTRICTION_2_2, /* 1790 */
+    XML_SCHEMAP_DERIVATION_OK_RESTRICTION_3, /* 1791 */
+    XML_SCHEMAP_WILDCARD_INVALID_NS_MEMBER, /* 1792 */
+    XML_SCHEMAP_INTERSECTION_NOT_EXPRESSIBLE, /* 1793 */
+    XML_SCHEMAP_UNION_NOT_EXPRESSIBLE, /* 1794 */
+    XML_SCHEMAP_SRC_IMPORT_3_1, /* 1795 */
+    XML_SCHEMAP_SRC_IMPORT_3_2, /* 1796 */
+    XML_SCHEMAP_DERIVATION_OK_RESTRICTION_4_1, /* 1797 */
+    XML_SCHEMAP_DERIVATION_OK_RESTRICTION_4_2, /* 1798 */
+    XML_SCHEMAP_DERIVATION_OK_RESTRICTION_4_3, /* 1799 */
+    XML_SCHEMAP_COS_CT_EXTENDS_1_3, /* 1800 */
+    XML_SCHEMAV_NOROOT = 1801,
+    XML_SCHEMAV_UNDECLAREDELEM, /* 1802 */
+    XML_SCHEMAV_NOTTOPLEVEL, /* 1803 */
+    XML_SCHEMAV_MISSING, /* 1804 */
+    XML_SCHEMAV_WRONGELEM, /* 1805 */
+    XML_SCHEMAV_NOTYPE, /* 1806 */
+    XML_SCHEMAV_NOROLLBACK, /* 1807 */
+    XML_SCHEMAV_ISABSTRACT, /* 1808 */
+    XML_SCHEMAV_NOTEMPTY, /* 1809 */
+    XML_SCHEMAV_ELEMCONT, /* 1810 */
+    XML_SCHEMAV_HAVEDEFAULT, /* 1811 */
+    XML_SCHEMAV_NOTNILLABLE, /* 1812 */
+    XML_SCHEMAV_EXTRACONTENT, /* 1813 */
+    XML_SCHEMAV_INVALIDATTR, /* 1814 */
+    XML_SCHEMAV_INVALIDELEM, /* 1815 */
+    XML_SCHEMAV_NOTDETERMINIST, /* 1816 */
+    XML_SCHEMAV_CONSTRUCT, /* 1817 */
+    XML_SCHEMAV_INTERNAL, /* 1818 */
+    XML_SCHEMAV_NOTSIMPLE, /* 1819 */
+    XML_SCHEMAV_ATTRUNKNOWN, /* 1820 */
+    XML_SCHEMAV_ATTRINVALID, /* 1821 */
+    XML_SCHEMAV_VALUE, /* 1822 */
+    XML_SCHEMAV_FACET, /* 1823 */
+    XML_SCHEMAV_CVC_DATATYPE_VALID_1_2_1, /* 1824 */
+    XML_SCHEMAV_CVC_DATATYPE_VALID_1_2_2, /* 1825 */
+    XML_SCHEMAV_CVC_DATATYPE_VALID_1_2_3, /* 1826 */
+    XML_SCHEMAV_CVC_TYPE_3_1_1, /* 1827 */
+    XML_SCHEMAV_CVC_TYPE_3_1_2, /* 1828 */
+    XML_SCHEMAV_CVC_FACET_VALID, /* 1829 */
+    XML_SCHEMAV_CVC_LENGTH_VALID, /* 1830 */
+    XML_SCHEMAV_CVC_MINLENGTH_VALID, /* 1831 */
+    XML_SCHEMAV_CVC_MAXLENGTH_VALID, /* 1832 */
+    XML_SCHEMAV_CVC_MININCLUSIVE_VALID, /* 1833 */
+    XML_SCHEMAV_CVC_MAXINCLUSIVE_VALID, /* 1834 */
+    XML_SCHEMAV_CVC_MINEXCLUSIVE_VALID, /* 1835 */
+    XML_SCHEMAV_CVC_MAXEXCLUSIVE_VALID, /* 1836 */
+    XML_SCHEMAV_CVC_TOTALDIGITS_VALID, /* 1837 */
+    XML_SCHEMAV_CVC_FRACTIONDIGITS_VALID, /* 1838 */
+    XML_SCHEMAV_CVC_PATTERN_VALID, /* 1839 */
+    XML_SCHEMAV_CVC_ENUMERATION_VALID, /* 1840 */
+    XML_SCHEMAV_CVC_COMPLEX_TYPE_2_1, /* 1841 */
+    XML_SCHEMAV_CVC_COMPLEX_TYPE_2_2, /* 1842 */
+    XML_SCHEMAV_CVC_COMPLEX_TYPE_2_3, /* 1843 */
+    XML_SCHEMAV_CVC_COMPLEX_TYPE_2_4, /* 1844 */
+    XML_SCHEMAV_CVC_ELT_1, /* 1845 */
+    XML_SCHEMAV_CVC_ELT_2, /* 1846 */
+    XML_SCHEMAV_CVC_ELT_3_1, /* 1847 */
+    XML_SCHEMAV_CVC_ELT_3_2_1, /* 1848 */
+    XML_SCHEMAV_CVC_ELT_3_2_2, /* 1849 */
+    XML_SCHEMAV_CVC_ELT_4_1, /* 1850 */
+    XML_SCHEMAV_CVC_ELT_4_2, /* 1851 */
+    XML_SCHEMAV_CVC_ELT_4_3, /* 1852 */
+    XML_SCHEMAV_CVC_ELT_5_1_1, /* 1853 */
+    XML_SCHEMAV_CVC_ELT_5_1_2, /* 1854 */
+    XML_SCHEMAV_CVC_ELT_5_2_1, /* 1855 */  
+    XML_SCHEMAV_CVC_ELT_5_2_2_1, /* 1856 */
+    XML_SCHEMAV_CVC_ELT_5_2_2_2_1, /* 1857 */
+    XML_SCHEMAV_CVC_ELT_5_2_2_2_2, /* 1858 */
+    XML_SCHEMAV_CVC_ELT_6, /* 1859 */
+    XML_SCHEMAV_CVC_ELT_7, /* 1860 */
+    XML_SCHEMAV_CVC_ATTRIBUTE_1, /* 1861 */
+    XML_SCHEMAV_CVC_ATTRIBUTE_2, /* 1862 */
+    XML_SCHEMAV_CVC_ATTRIBUTE_3, /* 1863 */
+    XML_SCHEMAV_CVC_ATTRIBUTE_4, /* 1864 */
+    XML_SCHEMAV_CVC_COMPLEX_TYPE_3_1, /* 1865 */
+    XML_SCHEMAV_CVC_COMPLEX_TYPE_3_2_1, /* 1866 */
+    XML_SCHEMAV_CVC_COMPLEX_TYPE_3_2_2, /* 1867 */
+    XML_SCHEMAV_CVC_COMPLEX_TYPE_4, /* 1868 */
+    XML_SCHEMAV_CVC_COMPLEX_TYPE_5_1, /* 1869 */
+    XML_SCHEMAV_CVC_COMPLEX_TYPE_5_2, /* 1870 */
+    XML_SCHEMAV_ELEMENT_CONTENT, /* 1871 */
+    XML_SCHEMAV_DOCUMENT_ELEMENT_MISSING, /* 1872 */
+    XML_SCHEMAV_CVC_COMPLEX_TYPE_1, /* 1873 */
+    XML_SCHEMAV_CVC_AU, /* 1874 */
+    XML_SCHEMAV_CVC_TYPE_1, /* 1875 */
+    XML_SCHEMAV_CVC_TYPE_2, /* 1876 */
+    XML_SCHEMAV_CVC_IDC, /* 1877 */
+    XML_SCHEMAV_CVC_WILDCARD, /* 1878 */
     XML_XPTR_UNKNOWN_SCHEME = 1900, 
     XML_XPTR_CHILDSEQ_START, /* 1901 */
     XML_XPTR_EVAL_FAILED, /* 1902 */
@@ -657,6 +666,7 @@ typedef enum {
     XML_FTP_PASV_ANSWER = 2000,
     XML_FTP_EPSV_ANSWER, /* 2001 */
     XML_FTP_ACCNT, /* 2002 */
+    XML_FTP_URL_SYNTAX, /* 2003 */
     XML_HTTP_URL_SYNTAX = 2020,
     XML_HTTP_USE_IP, /* 2021 */
     XML_HTTP_UNKNOWN_HOST, /* 2022 */
@@ -718,19 +728,72 @@ typedef enum {
     XML_SCHEMAP_SRC_ATTRIBUTE_4, /* 3055 */
     XML_SCHEMAP_NO_XMLNS, /* 3056 */
     XML_SCHEMAP_NO_XSI, /* 3057 */      
-    XML_SCHEMAP_COS_VALID_DEFAULT_1,
-    XML_SCHEMAP_COS_VALID_DEFAULT_2_1,
-    XML_SCHEMAP_COS_VALID_DEFAULT_2_2_1,
-    XML_SCHEMAP_COS_VALID_DEFAULT_2_2_2,
-    XML_SCHEMAP_CVC_SIMPLE_TYPE,
-    XML_SCHEMAP_COS_CT_EXTENDS_1_1,
-    XML_SCHEMAP_SRC_IMPORT_1_1,
-    XML_SCHEMAP_SRC_IMPORT_1_2,
-    XML_SCHEMAP_SRC_IMPORT_2,
-    XML_SCHEMAP_SRC_IMPORT_2_1,
-    XML_SCHEMAP_SRC_IMPORT_2_2,
-    XML_SCHEMAP_INTERNAL, /* non-W3C */
-    XML_SCHEMAP_NOT_DETERMINISTIC /* non-W3C */
+    XML_SCHEMAP_COS_VALID_DEFAULT_1, /* 3058 */
+    XML_SCHEMAP_COS_VALID_DEFAULT_2_1, /* 3059 */
+    XML_SCHEMAP_COS_VALID_DEFAULT_2_2_1, /* 3060 */
+    XML_SCHEMAP_COS_VALID_DEFAULT_2_2_2, /* 3061 */
+    XML_SCHEMAP_CVC_SIMPLE_TYPE, /* 3062 */
+    XML_SCHEMAP_COS_CT_EXTENDS_1_1, /* 3063 */
+    XML_SCHEMAP_SRC_IMPORT_1_1, /* 3064 */
+    XML_SCHEMAP_SRC_IMPORT_1_2, /* 3065 */
+    XML_SCHEMAP_SRC_IMPORT_2, /* 3066 */
+    XML_SCHEMAP_SRC_IMPORT_2_1, /* 3067 */
+    XML_SCHEMAP_SRC_IMPORT_2_2, /* 3068 */
+    XML_SCHEMAP_INTERNAL, /* 3069 non-W3C */
+    XML_SCHEMAP_NOT_DETERMINISTIC, /* 3070 non-W3C */
+    XML_SCHEMAP_SRC_ATTRIBUTE_GROUP_1, /* 3071 */
+    XML_SCHEMAP_SRC_ATTRIBUTE_GROUP_2, /* 3072 */
+    XML_SCHEMAP_SRC_ATTRIBUTE_GROUP_3, /* 3073 */
+    XML_SCHEMAP_MG_PROPS_CORRECT_1, /* 3074 */
+    XML_SCHEMAP_MG_PROPS_CORRECT_2, /* 3075 */
+    XML_SCHEMAP_SRC_CT_1, /* 3076 */
+    XML_SCHEMAP_DERIVATION_OK_RESTRICTION_2_1_3, /* 3077 */
+    XML_SCHEMAP_AU_PROPS_CORRECT_2, /* 3078 */
+    XML_SCHEMAP_A_PROPS_CORRECT_2, /* 3079 */
+    XML_MODULE_OPEN = 4900, /* 4900 */
+    XML_MODULE_CLOSE, /* 4901 */
+    XML_CHECK_FOUND_ELEMENT = 5000,
+    XML_CHECK_FOUND_ATTRIBUTE, /* 5001 */
+    XML_CHECK_FOUND_TEXT, /* 5002 */
+    XML_CHECK_FOUND_CDATA, /* 5003 */
+    XML_CHECK_FOUND_ENTITYREF, /* 5004 */
+    XML_CHECK_FOUND_ENTITY, /* 5005 */
+    XML_CHECK_FOUND_PI, /* 5006 */
+    XML_CHECK_FOUND_COMMENT, /* 5007 */
+    XML_CHECK_FOUND_DOCTYPE, /* 5008 */
+    XML_CHECK_FOUND_FRAGMENT, /* 5009 */
+    XML_CHECK_FOUND_NOTATION, /* 5010 */
+    XML_CHECK_UNKNOWN_NODE, /* 5011 */
+    XML_CHECK_ENTITY_TYPE, /* 5012 */
+    XML_CHECK_NO_PARENT, /* 5013 */
+    XML_CHECK_NO_DOC, /* 5014 */
+    XML_CHECK_NO_NAME, /* 5015 */
+    XML_CHECK_NO_ELEM, /* 5016 */
+    XML_CHECK_WRONG_DOC, /* 5017 */
+    XML_CHECK_NO_PREV, /* 5018 */
+    XML_CHECK_WRONG_PREV, /* 5019 */
+    XML_CHECK_NO_NEXT, /* 5020 */
+    XML_CHECK_WRONG_NEXT, /* 5021 */
+    XML_CHECK_NOT_DTD, /* 5022 */
+    XML_CHECK_NOT_ATTR, /* 5023 */
+    XML_CHECK_NOT_ATTR_DECL, /* 5024 */
+    XML_CHECK_NOT_ELEM_DECL, /* 5025 */
+    XML_CHECK_NOT_ENTITY_DECL, /* 5026 */
+    XML_CHECK_NOT_NS_DECL, /* 5027 */
+    XML_CHECK_NO_HREF, /* 5028 */
+    XML_CHECK_WRONG_PARENT,/* 5029 */
+    XML_CHECK_NS_SCOPE, /* 5030 */
+    XML_CHECK_NS_ANCESTOR, /* 5031 */
+    XML_CHECK_NOT_UTF8, /* 5032 */
+    XML_CHECK_NO_DICT, /* 5033 */
+    XML_CHECK_NOT_NCNAME, /* 5034 */
+    XML_CHECK_OUTSIDE_DICT, /* 5035 */
+    XML_CHECK_WRONG_NAME, /* 5036 */
+    XML_CHECK_NAME_NOT_NULL /* 5037 */
+#if 0
+    XML_CHECK_, /* 5033 */
+    XML_CHECK_X /* 503 */
+#endif
 } xmlParserErrors;
 
 /**
@@ -829,7 +892,7 @@ XMLPUBFUN void XMLCALL
 				 const char *str2,
 				 const char *str3,
 				 int int1,
-				 int int2,
+				 int col,
 				 const char *msg,
 				 ...);
 XMLPUBFUN void XMLCALL 
