@@ -29,28 +29,28 @@ XMLPUBFUN void XMLCALL xmlCheckVersion(int version);
  *
  * the version string like "1.2.3"
  */
-#define LIBXML_DOTTED_VERSION "2.6.15"
+#define LIBXML_DOTTED_VERSION "2.6.18"
 
 /**
  * LIBXML_VERSION:
  *
  * the version number: 1.2.3 value is 1002003
  */
-#define LIBXML_VERSION 20615
+#define LIBXML_VERSION 20618
 
 /**
  * LIBXML_VERSION_STRING:
  *
  * the version number string, 1.2.3 value is "1002003"
  */
-#define LIBXML_VERSION_STRING "20615"
+#define LIBXML_VERSION_STRING "20618"
 
 /**
  * LIBXML_VERSION_EXTRA:
  *
  * extra version information, used to show a CVS compilation
  */
-#define LIBXML_VERSION_EXTRA "-CVS2214"
+#define LIBXML_VERSION_EXTRA "-CVS2374"
 
 /**
  * LIBXML_TEST_VERSION:
@@ -58,7 +58,7 @@ XMLPUBFUN void XMLCALL xmlCheckVersion(int version);
  * Macro to check that the libxml version in use is compatible with
  * the version the software has been compiled against
  */
-#define LIBXML_TEST_VERSION xmlCheckVersion(20615);
+#define LIBXML_TEST_VERSION xmlCheckVersion(20618);
 
 #ifndef VMS
 #if 0
@@ -339,6 +339,16 @@ XMLPUBFUN void XMLCALL xmlCheckVersion(int version);
 #define LIBXML_SCHEMAS_ENABLED
 #endif
 
+/**
+ * LIBXML_MODULES_ENABLED:
+ *
+ * Whether the module interfaces are compiled in
+ */
+#if 1
+#define LIBXML_MODULES_ENABLED
+#define LIBXML_MODULE_EXTENSION ".so" 
+#endif
+
 /**
  * ATTRIBUTE_UNUSED:
  *
