@@ -327,14 +327,16 @@ static void
 xmlSaveCtxtInit(xmlSaveCtxtPtr ctxt)
 {
     int i;
+    int len;
 
     if (ctxt == NULL) return;
     if ((ctxt->encoding == NULL) && (ctxt->escape == NULL))
         ctxt->escape = xmlEscapeEntities;
-    if (xmlTreeIndentString == NULL) {
+    len = xmlStrlen((xmlChar *)xmlTreeIndentString);
+    if ((xmlTreeIndentString == NULL) || (len == 0)) {
         memset(&ctxt->indent[0], 0, MAX_INDENT + 1);
     } else {
-	ctxt->indent_size = xmlStrlen((const xmlChar *) xmlTreeIndentString);
+	ctxt->indent_size = len;
 	ctxt->indent_nr = MAX_INDENT / ctxt->indent_size;
 	for (i = 0;i < ctxt->indent_nr;i++)
 	    memcpy(&ctxt->indent[i * ctxt->indent_size], xmlTreeIndentString,
@@ -377,7 +379,14 @@ xmlNewSaveCtxt(const char *encoding, int options)
 	return ( NULL );
     }
     memset(ret, 0, sizeof(xmlSaveCtxt));
+
+    /*
+     * Use the options
+     */
     ret->options = options;
+    if (options & XML_SAVE_FORMAT)
+        ret->format = 1;
+
     if (encoding != NULL) {
         ret->handler = xmlFindCharEncodingHandler(encoding);
 	if (ret->handler == NULL) {
@@ -663,9 +672,7 @@ xmlNodeDumpOutputInternal(xmlSaveCtxtPtr ctxt, xmlNodePtr cur) {
     }
     if (cur->type == XML_TEXT_NODE) {
 	if (cur->content != NULL) {
-	    if ((cur->name == xmlStringText) ||
-		(cur->name != xmlStringTextNoenc)) {
-
+	    if (cur->name != xmlStringTextNoenc) {
                 xmlOutputBufferWriteEscape(buf, cur->content, ctxt->escape);
 	    } else {
 		/*
